import base64
from pathlib import Path
from typing import Annotated, cast
from urllib.parse import urlparse

import httpx
from pandas import DataFrame, json_normalize

from impresso.api_client.api.tools import (
    perform_image_embedding,
    perform_ner,
    perform_text_embedding,
)
from impresso.api_client.models.impresso_embedding_response import (
    ImpressoEmbeddingResponse,
)
from impresso.api_client.models.impresso_image_embedding_request import (
    ImpressoImageEmbeddingRequest,
)
from impresso.api_client.models.impresso_image_embedding_request_search_target import (
    ImpressoImageEmbeddingRequestSearchTarget,
    ImpressoImageEmbeddingRequestSearchTargetLiteral,
)
from impresso.api_client.models.impresso_named_entity_recognition_request import (
    ImpressoNamedEntityRecognitionRequest,
)
from impresso.api_client.models.impresso_named_entity_recognition_request_method import (
    ImpressoNamedEntityRecognitionRequestMethod,
)
from impresso.api_client.models.impresso_text_embedding_request import (
    ImpressoTextEmbeddingRequest,
)
from impresso.api_client.models.impresso_text_embedding_request_search_target import (
    ImpressoTextEmbeddingRequestSearchTarget,
    ImpressoTextEmbeddingRequestSearchTargetLiteral,
)
from impresso.api_client.types import UNSET
from impresso.api_models import ImpressoNerResponse
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error
from impresso.util.py import get_enum_from_literal

Base64Str = Annotated[str, "base64-encoded string"]
Embedding = Annotated[str, "base64-encoded string with model prefix"]


def is_url(string: str) -> bool:
    """Check if a string is a valid URL."""
    try:
        parsed = urlparse(string)
        return parsed.scheme in ("http", "https", "ftp", "ftps") and bool(parsed.netloc)
    except Exception:
        return False


def is_file(string: str) -> bool:
    """Check if a string is a valid file path."""
    try:
        path = Path(string)
        return path.exists() and path.is_file()
    except Exception:
        return False


class ImpressoNerSchema(ImpressoNerResponse):
    """Schema for the find entities response."""

    pass


class NerContainer(DataContainer):
    """Name entity recognition result container."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["entities"]
        if len(data):
            return json_normalize(self._data.to_dict()["entities"]).set_index("id")
        return DataFrame()

    @property
    def total(self) -> int:
        """Total number of results."""
        return len(self.raw.get("entities", []))

    @property
    def limit(self) -> int:
        """Page size."""
        return len(self.raw.get("entities", []))

    @property
    def offset(self) -> int:
        """Page offset."""
        return 0

    @property
    def size(self) -> int:
        """Current page size."""
        return len(self.raw.get("entities", []))


class ToolsResource(Resource):
    """Various helper tools for text processing and embedding generation.

    Examples:
        Extract named entities from text:
        >>> entities = tools.ner("Napoleon visited Paris in 1815.")  # doctest: +SKIP
        >>> print(entities.df)  # doctest: +SKIP

        Extract and link entities to Wikidata:
        >>> entities = tools.ner_nel("Napoleon visited Paris in 1815.")  # doctest: +SKIP
        >>> print(entities.df)  # doctest: +SKIP

        Generate text embedding for semantic search:
        >>> embedding = tools.embed_text("military conflict", target="text")  # doctest: +SKIP
        >>> results = search.find(embedding=embedding)  # doctest: +SKIP

        Generate image embedding from file:
        >>> embedding = tools.embed_image("path/to/image.jpg", target="image")  # doctest: +SKIP
        >>> similar_images = images.find(embedding=embedding)  # doctest: +SKIP

        Generate image embedding from URL:
        >>> embedding = tools.embed_image("https://example.com/image.jpg", target="image")  # doctest: +SKIP
        >>> similar_images = images.find(embedding=embedding)  # doctest: +SKIP
    """

    name = "tools"

    def ner(self, text: str) -> NerContainer:
        """Named Entity Recognition

        This method is faster than `ner_nel` but does not provide any linking to external resources.

        Args:
            text: Text to process

        Returns:
            NerContainer: List of named entities
        """
        result = perform_ner.sync(
            client=self._api_client,
            body=ImpressoNamedEntityRecognitionRequest(
                text=text, method=ImpressoNamedEntityRecognitionRequestMethod.NER
            ),
        )
        raise_for_error(result)

        return NerContainer(
            result,
            ImpressoNerSchema,
            web_app_search_result_url=None,
        )

    def ner_nel(self, text: str) -> NerContainer:
        """Named Entity Recognition and Named Entity Linking

        This method is slower than `ner` but provides linking to external resources.

        Args:
            text: Text to process

        Returns:
            NerContainer: List of named entities
        """
        result = perform_ner.sync(
            client=self._api_client,
            body=ImpressoNamedEntityRecognitionRequest(
                text=text, method=ImpressoNamedEntityRecognitionRequestMethod.NER_NEL
            ),
        )
        raise_for_error(result)

        return NerContainer(
            result,
            ImpressoNerSchema,
            web_app_search_result_url=None,
        )

    def nel(self, text: str) -> NerContainer:
        """Named Entity Linking

        This method requires named entities to be enclosed in tags: [START]entity[END].

        Args:
            text: Text to process

        Returns:
            NerContainer: List of named entities
        """
        result = perform_ner.sync(
            client=self._api_client,
            body=ImpressoNamedEntityRecognitionRequest(
                text=text, method=ImpressoNamedEntityRecognitionRequestMethod.NEL
            ),
        )
        raise_for_error(result)

        return NerContainer(
            result,
            ImpressoNerSchema,
            web_app_search_result_url=None,
        )

    def embed_image(
        self,
        image: bytes | Base64Str | str,
        target: ImpressoImageEmbeddingRequestSearchTargetLiteral,
    ) -> Embedding:
        """Embed an image into a vector space.

        Args:
            image: Image to embed. Can be raw bytes, a base64-encoded string, a URL of an image,
                or a path to a file.
            target: Target collection to embed the image into. Currently, only "image" is supported.

        Returns:
            Embedding: The image embedding as a base64 string prefixed with model tag.
        """
        image_as_base64: str
        if isinstance(image, bytes):
            image_as_base64 = base64.b64encode(image).decode("utf-8")
        elif is_file(image):
            with open(image, "rb") as file:
                image_as_base64 = base64.b64encode(file.read()).decode("utf-8")
        elif is_url(image):
            response = httpx.get(image)
            response.raise_for_status()
            image_as_base64 = base64.b64encode(response.content).decode("utf-8")
        else:
            image_as_base64 = image  # assume it's already a base64-encoded string

        search_target = get_enum_from_literal(
            target,
            ImpressoImageEmbeddingRequestSearchTarget,
        )
        if search_target == UNSET:
            raise ValueError(f"Invalid search target: {target}")

        result = perform_image_embedding.sync(
            client=self._api_client,
            body=ImpressoImageEmbeddingRequest(
                bytes_=image_as_base64,
                search_target=cast(
                    ImpressoImageEmbeddingRequestSearchTarget, search_target
                ),
            ),
        )
        raise_for_error(result)
        if isinstance(result, ImpressoEmbeddingResponse):
            return cast(str, result.embedding)
        raise ValueError("Unexpected response format")

    def embed_text(
        self,
        text: str,
        target: ImpressoTextEmbeddingRequestSearchTargetLiteral,
    ) -> Embedding:
        """Embed text into a vector space.

        Args:
            text: Text to embed.
            target: Target collection to embed the text into.

        Returns:
            Embedding: The text embedding as a base64 string prefixed with model tag.
        """
        search_target = get_enum_from_literal(
            target,
            ImpressoTextEmbeddingRequestSearchTarget,
        )
        if search_target == UNSET:
            raise ValueError(f"Invalid search target: {target}")

        result = perform_text_embedding.sync(
            client=self._api_client,
            body=ImpressoTextEmbeddingRequest(
                text=text,
                search_target=cast(
                    ImpressoTextEmbeddingRequestSearchTarget, search_target
                ),
            ),
        )
        raise_for_error(result)
        if isinstance(result, ImpressoEmbeddingResponse):
            return cast(Embedding, result.embedding)
        raise ValueError("Unexpected response format")



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="EntityMention")


@_attrs_define
class EntityMention:
    """An entity (location, person, etc.) mentioned in the content item.

    Attributes:
        surface_form (str): The surface form (label) of the entity mention
        mention_confidence (float): Confidence score of the entity mention
        start_offset (Union[Unset, int]): Start offset of the entity mention in the content item
        end_offset (Union[Unset, int]): End offset of the entity mention in the content item
    """

    surface_form: str
    mention_confidence: float
    start_offset: Union[Unset, int] = UNSET
    end_offset: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        surface_form = self.surface_form

        mention_confidence = self.mention_confidence

        start_offset = self.start_offset

        end_offset = self.end_offset

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "surfaceForm": surface_form,
                "mentionConfidence": mention_confidence,
            }
        )
        if start_offset is not UNSET:
            field_dict["startOffset"] = start_offset
        if end_offset is not UNSET:
            field_dict["endOffset"] = end_offset

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        surface_form = d.pop("surfaceForm")

        mention_confidence = d.pop("mentionConfidence")

        start_offset = d.pop("startOffset", UNSET)

        end_offset = d.pop("endOffset", UNSET)

        entity_mention = cls(
            surface_form=surface_form,
            mention_confidence=mention_confidence,
            start_offset=start_offset,
            end_offset=end_offset,
        )

        return entity_mention



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="NamedEntity")


@_attrs_define
class NamedEntity:
    """An named entity (location, persion, etc) present in text.

    Attributes:
        uid (str): Unique identifier of the entity
        count (Union[Unset, float]): How many times it is mentioned in the text
    """

    uid: str
    count: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        count = self.count

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        count = d.pop("count", UNSET)

        named_entity = cls(
            uid=uid,
            count=count,
        )

        return named_entity



import datetime
from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.wikidata_person_type import WikidataPersonType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.wikidata_location import WikidataLocation
    from ..models.wikidata_person_descriptions import WikidataPersonDescriptions
    from ..models.wikidata_person_labels import WikidataPersonLabels


T = TypeVar("T", bound="WikidataPerson")


@_attrs_define
class WikidataPerson:
    """Wikidata person schema. Based on https://schema.org/Person

    Attributes:
        id (str): The Q Wikidata ID of the person (https://www.wikidata.org/wiki/Wikidata:Identifiers)
        type (WikidataPersonType): The type of the entity
        labels (Union[Unset, WikidataPersonLabels]): Labels of the person in different languages
        descriptions (Union[Unset, WikidataPersonDescriptions]): Descriptions of the person in different languages
        birth_date (Union[Unset, datetime.datetime]): The birth date of the person
        death_date (Union[Unset, datetime.datetime]): The death date of the person
        birth_place (Union[Unset, WikidataLocation]): Wikidata location schema. Based on https://schema.org/Place
        death_place (Union[Unset, WikidataLocation]): Wikidata location schema. Based on https://schema.org/Place
    """

    id: str
    type: WikidataPersonType
    labels: Union[Unset, "WikidataPersonLabels"] = UNSET
    descriptions: Union[Unset, "WikidataPersonDescriptions"] = UNSET
    birth_date: Union[Unset, datetime.datetime] = UNSET
    death_date: Union[Unset, datetime.datetime] = UNSET
    birth_place: Union[Unset, "WikidataLocation"] = UNSET
    death_place: Union[Unset, "WikidataLocation"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        type = self.type.value

        labels: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels.to_dict()

        descriptions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.descriptions, Unset):
            descriptions = self.descriptions.to_dict()

        birth_date: Union[Unset, str] = UNSET
        if not isinstance(self.birth_date, Unset):
            birth_date = self.birth_date.isoformat()

        death_date: Union[Unset, str] = UNSET
        if not isinstance(self.death_date, Unset):
            death_date = self.death_date.isoformat()

        birth_place: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.birth_place, Unset):
            birth_place = self.birth_place.to_dict()

        death_place: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.death_place, Unset):
            death_place = self.death_place.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "type": type,
            }
        )
        if labels is not UNSET:
            field_dict["labels"] = labels
        if descriptions is not UNSET:
            field_dict["descriptions"] = descriptions
        if birth_date is not UNSET:
            field_dict["birthDate"] = birth_date
        if death_date is not UNSET:
            field_dict["deathDate"] = death_date
        if birth_place is not UNSET:
            field_dict["birthPlace"] = birth_place
        if death_place is not UNSET:
            field_dict["deathPlace"] = death_place

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.wikidata_location import WikidataLocation
        from ..models.wikidata_person_descriptions import WikidataPersonDescriptions
        from ..models.wikidata_person_labels import WikidataPersonLabels

        d = src_dict.copy()
        id = d.pop("id")

        type = WikidataPersonType(d.pop("type"))

        _labels = d.pop("labels", UNSET)
        labels: Union[Unset, WikidataPersonLabels]
        if isinstance(_labels, Unset):
            labels = UNSET
        else:
            labels = WikidataPersonLabels.from_dict(_labels)

        _descriptions = d.pop("descriptions", UNSET)
        descriptions: Union[Unset, WikidataPersonDescriptions]
        if isinstance(_descriptions, Unset):
            descriptions = UNSET
        else:
            descriptions = WikidataPersonDescriptions.from_dict(_descriptions)

        _birth_date = d.pop("birthDate", UNSET)
        birth_date: Union[Unset, datetime.datetime]
        if isinstance(_birth_date, Unset):
            birth_date = UNSET
        else:
            birth_date = isoparse(_birth_date)

        _death_date = d.pop("deathDate", UNSET)
        death_date: Union[Unset, datetime.datetime]
        if isinstance(_death_date, Unset):
            death_date = UNSET
        else:
            death_date = isoparse(_death_date)

        _birth_place = d.pop("birthPlace", UNSET)
        birth_place: Union[Unset, WikidataLocation]
        if isinstance(_birth_place, Unset):
            birth_place = UNSET
        else:
            birth_place = WikidataLocation.from_dict(_birth_place)

        _death_place = d.pop("deathPlace", UNSET)
        death_place: Union[Unset, WikidataLocation]
        if isinstance(_death_place, Unset):
            death_place = UNSET
        else:
            death_place = WikidataLocation.from_dict(_death_place)

        wikidata_person = cls(
            id=id,
            type=type,
            labels=labels,
            descriptions=descriptions,
            birth_date=birth_date,
            death_date=death_date,
            birth_place=birth_place,
            death_place=death_place,
        )

        return wikidata_person



from enum import Enum
from typing import Literal


class FindMediaSourcesType(str, Enum):
    NEWSPAPER = "newspaper"

    def __str__(self) -> str:
        return str(self.value)


FindMediaSourcesTypeLiteral = Literal["newspaper",]



from enum import Enum
from typing import Literal


class EntityDetailsType(str, Enum):
    LOCATION = "location"
    NEWSAGENCY = "newsagency"
    ORGANISATION = "organisation"
    PERSON = "person"

    def __str__(self) -> str:
        return str(self.value)


EntityDetailsTypeLiteral = Literal[
    "location",
    "newsagency",
    "organisation",
    "person",
]



from typing import Any, Callable, Iterator, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.search_facets import get_tr_clusters_facet
from impresso.api_client.api.text_reuse_clusters import find_text_reuse_clusters
from impresso.api_client.models.find_text_reuse_clusters_order_by import (
    FindTextReuseClustersOrderBy,
    FindTextReuseClustersOrderByLiteral,
)
from impresso.api_client.models.get_tr_clusters_facet_id import (
    GetTrClustersFacetId,
    GetTrClustersFacetIdLiteral,
)
from impresso.api_client.models.get_tr_clusters_facet_order_by import (
    GetTrClustersFacetOrderBy,
    GetTrClustersFacetOrderByLiteral,
)
from impresso.api_client.types import UNSET, Unset
from impresso.api_client.models.get_search_facet_base_find_response import (
    GetSearchFacetBaseFindResponse,
)
from impresso.api_client.models.find_text_reuse_clusters_base_find_response import (
    FindTextReuseClustersBaseFindResponse,
)
from impresso.api_models import (
    BaseFind,
    Filter,
    Q,
    TextReuseCluster,
)
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.resources.search import (
    FacetDataContainer,
    FacetResponseSchema,
)
from impresso.structures import AND, OR, DateRange
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal


class FindTextReuseClusterResponseSchema(BaseFind):
    """Schema for the text reuse clusters response."""

    data: list[TextReuseCluster]


class FindTextReuseClustersContainer(DataContainer):
    def __init__(
        self,
        data: FindTextReuseClustersBaseFindResponse,
        pydantic_model: type[FindTextReuseClusterResponseSchema],
        fetch_method: Callable[..., "FindTextReuseClustersContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(data).set_index("uid")
        return DataFrame()

    def pages(self) -> Iterator["FindTextReuseClustersContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


Range = tuple[int, int]


class TextReuseClustersResource(Resource):
    """
    Interact with the text reuse clusters endpoint of the Impresso API.

    This resource allows searching for text reuse clusters based on various criteria
    and retrieving facet information about these clusters.

    Examples:
        Find clusters with size between 10 and 20:
        >>> results = textReuseClusters.find(cluster_size=(10, 20)) # doctest: +SKIP
        >>> print(results.df) # doctest: +SKIP

        Get the distribution of newspapers involved in clusters:
        >>> facet_results = textReuseClusters.facet(facet='newspaper', order_by='count') # doctest: +SKIP
        >>> print(facet_results.df) # doctest: +SKIP
    """

    name = "textReuseClusters"

    def find(
        self,
        term: str | None = None,
        title: str | AND[str] | OR[str] | None = None,
        order_by: FindTextReuseClustersOrderByLiteral | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        limit: int | None = None,
        offset: int | None = None,
        front_page: bool | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        language: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
    ) -> FindTextReuseClustersContainer:
        """
        Find text reuse clusters based on various criteria.

        Args:
            term: Search for clusters containing specific text.
            title: Filter clusters by the title of the articles within them.
            order_by: Specify the sorting order for the results.
            cluster_size: Filter clusters by the number of items they contain.
            lexical_overlap: Filter clusters by the lexical overlap score.
            day_delta: Filter clusters by the time span (in days) between the first and last item.
            date_range: Filter clusters based on the date range of their items.
            newspaper_id: Filter clusters containing items from specific newspapers.
            collection_id: Filter clusters containing items from specific collections.
            limit: Maximum number of clusters to return.
            offset: Number of clusters to skip from the beginning.
            front_page: Filter clusters containing items published on the front page.
            topic_id: Filter clusters associated with specific topics.
            language: Filter clusters by the language of their items.
            country: Filter clusters by the country of publication of their items.
            mention: Filter clusters containing specific mentions (named entities).
            entity_id: Filter clusters associated with specific entity IDs.

        Returns:
            FindTextReuseClustersContainer: A container holding the search results.

        Examples:
            Find clusters with size between 10 and 20:
            >>> results = textReuseClusters.find(cluster_size=(10, 20)) # doctest: +SKIP
            >>> print(results.df) # doctest: +SKIP

            Find clusters related to 'politics' in Swiss newspapers:
            >>> results = textReuseClusters.find(term='politics', country='CH') # doctest: +SKIP
            >>> print(results.df) # doctest: +SKIP
        """

        filters = _build_filters(
            text=term,
            cluster_size=cluster_size,
            title=title,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
            collection_id=collection_id,
            front_page=front_page,
            topic_id=topic_id,
            language=language,
            country=country,
            mention=mention,
            entity_id=entity_id,
        )
        filters_pb = filters_as_protobuf(filters or [])

        result = find_text_reuse_clusters.sync(
            client=self._api_client,
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindTextReuseClustersOrderBy)
                if order_by is not None
                else UNSET
            ),
            filters=filters_pb if filters_pb else UNSET,
        )
        raise_for_error(result)
        return FindTextReuseClustersContainer(
            data=cast(FindTextReuseClustersBaseFindResponse, result),
            pydantic_model=FindTextReuseClusterResponseSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "title": title,
                "order_by": order_by,
                "cluster_size": cluster_size,
                "lexical_overlap": lexical_overlap,
                "day_delta": day_delta,
                "date_range": date_range,
                "newspaper_id": newspaper_id,
                "collection_id": collection_id,
                "front_page": front_page,
                "topic_id": topic_id,
                "language": language,
                "country": country,
                "mention": mention,
                "entity_id": entity_id,
            },
            web_app_search_result_url=_build_web_app_find_clusters_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )

    def facet(
        self,
        facet: GetTrClustersFacetIdLiteral,
        order_by: GetTrClustersFacetOrderByLiteral | None = "value",
        limit: int | None = None,
        offset: int | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
    ) -> FacetDataContainer:
        """
        Get facet information for text reuse clusters based on specified filters.

        Facets provide aggregated counts for different properties of the clusters,
        such as the distribution of cluster sizes or newspapers involved.

        Args:
            facet: The specific facet to retrieve (e.g., 'newspaper', 'cluster_size').
            order_by: How to order the facet values (e.g., 'value', 'count').
            limit: Maximum number of facet values to return.
            offset: Number of facet values to skip.
            cluster_size: Filter clusters by size before calculating facets.
            date_range: Filter clusters by date range before calculating facets.
            newspaper_id: Filter clusters by newspaper before calculating facets.
            lexical_overlap: Filter clusters by lexical overlap before calculating facets.
            day_delta: Filter clusters by day delta before calculating facets.

        Returns:
            FacetDataContainer: A container holding the facet results.

        Examples:
            Get the top 10 newspapers involved in clusters:
            >>> facet_results = textReuseClusters.facet(facet='newspaper', limit=10, order_by='count') # doctest: +SKIP
            >>> print(facet_results.df) # doctest: +SKIP

            Get the distribution of cluster sizes for clusters within a specific date range:
            >>> from impresso.structures import DateRange
            >>> date_filter = DateRange(start="1900-01-01", end="1910-12-31")
            >>> facet_results = textReuseClusters.facet(facet='cluster_size', date_range=date_filter) # doctest: +SKIP
            >>> print(facet_results.df) # doctest: +SKIP
        """
        facet_id = get_enum_from_literal(facet, GetTrClustersFacetId)
        if isinstance(facet_id, Unset):
            raise ValueError(f"{facet} is not a valid value")

        filters = _build_cluster_facet_filters(
            cluster_size=cluster_size,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
        )

        filters_pb = filters_as_protobuf(filters or [])

        result = get_tr_clusters_facet.sync(
            client=self._api_client,
            id=facet_id,
            filters=filters_pb if filters_pb else UNSET,
            offset=offset if offset is not None else UNSET,
            limit=limit if limit is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, GetTrClustersFacetOrderBy)
                if order_by is not None
                else UNSET
            ),
        )
        raise_for_error(result)
        return FacetDataContainer(
            data=cast(GetSearchFacetBaseFindResponse, result),  # Cast result
            pydantic_model=FacetResponseSchema,  # Use FacetResponseSchema
            fetch_method=self.facet,
            fetch_method_args={
                "facet": facet,
                "order_by": order_by,
                "cluster_size": cluster_size,
                "date_range": date_range,
                "newspaper_id": newspaper_id,
                "lexical_overlap": lexical_overlap,
                "day_delta": day_delta,
            },
            web_app_search_result_url=_build_web_app_find_clusters_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )


def _build_cluster_facet_filters(
    cluster_size: Range | AND[Range] | OR[Range] | None = None,
    date_range: DateRange | None = None,
    newspaper_id: str | OR[str] | None = None,
    lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
    day_delta: Range | AND[Range] | OR[Range] | None = None,
) -> list[Filter]:
    """Build text reuse clusters facet filters."""

    filters: list[Filter] = []
    if cluster_size is not None:
        filters.extend(
            and_or_filter(
                cluster_size,
                "text_reuse_cluster_size",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if date_range is not None:
        filters.append(
            Filter(
                type="daterange",
                q=Q(DateRange._as_filter_value(date_range)),
                context="exclude" if date_range.inverted else "include",
                daterange=None,
            )
        )
    if newspaper_id is not None:
        filters.extend(and_or_filter(newspaper_id, "newspaper"))
    if lexical_overlap is not None:
        filters.extend(
            and_or_filter(
                lexical_overlap,
                "text_reuse_cluster_lexical_overlap",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if day_delta is not None:
        filters.extend(
            and_or_filter(
                day_delta,
                "text_reuse_cluster_day_delta",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    return filters


def _build_filters(
    text: str | None = None,
    cluster_id: str | AND[str] | OR[str] | None = None,
    cluster_size: Range | AND[Range] | OR[Range] | None = None,
    title: str | AND[str] | OR[str] | None = None,
    lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
    day_delta: Range | AND[Range] | OR[Range] | None = None,
    date_range: DateRange | None = None,
    newspaper_id: str | OR[str] | None = None,
    collection_id: str | OR[str] | None = None,
    front_page: bool | None = None,
    topic_id: str | AND[str] | OR[str] | None = None,
    language: str | OR[str] | None = None,
    country: str | OR[str] | None = None,
    mention: str | AND[str] | OR[str] | None = None,
    entity_id: str | AND[str] | OR[str] | None = None,
) -> list[Filter]:
    """Build text reuse clusters filters."""

    filters: list[Filter] = []
    if text is not None:
        filters.extend(and_or_filter(text, "string"))
    if cluster_id is not None:
        filters.extend(and_or_filter(cluster_id, "text_reuse_cluster"))
    if cluster_size is not None:
        filters.extend(
            and_or_filter(
                cluster_size,
                "text_reuse_cluster_size",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if title is not None:
        filters.extend(and_or_filter(title, "title"))
    if lexical_overlap is not None:
        filters.extend(
            and_or_filter(
                lexical_overlap,
                "text_reuse_cluster_lexical_overlap",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if day_delta is not None:
        filters.extend(
            and_or_filter(
                day_delta,
                "text_reuse_cluster_day_delta",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if date_range is not None:
        filters.append(
            Filter(
                type="daterange",
                q=Q(DateRange._as_filter_value(date_range)),
                context="exclude" if date_range.inverted else "include",
                daterange=None,
            )
        )
    if newspaper_id is not None:
        filters.extend(and_or_filter(newspaper_id, "newspaper"))
    if collection_id is not None:
        filters.extend(and_or_filter(collection_id, "collection"))
    if front_page:
        filters.append(Filter(type="is_front", daterange=None))
    if topic_id is not None:
        filters.extend(and_or_filter(topic_id, "topic"))
    if language is not None:
        filters.extend(and_or_filter(language, "language"))
    if country is not None:
        filters.extend(and_or_filter(country, "country"))
    if mention is not None:
        filters.extend(and_or_filter(mention, "mention"))
    if entity_id is not None:
        filters.extend(and_or_filter(entity_id, "entity"))

    return filters


def _build_web_app_find_clusters_url(
    base_url: str,
    filters=str | None,
    limit=int | None,
    offset=int | None,
    order_by=GetTrClustersFacetOrderByLiteral | None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "orderBy": order_by,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/text-reuse/clusters"
    return f"{url}?{query_string}" if query_string else url



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..models.filter_context import FilterContext
from ..models.filter_op import FilterOp
from ..models.filter_precision import FilterPrecision
from ..types import UNSET, Unset

T = TypeVar("T", bound="Filter")


@_attrs_define
class Filter:
    """A single filter criteria

    Attributes:
        type (str): Possible values are in 'search.validators:eachFilterValidator.type.choices'
        context (Union[Unset, FilterContext]):  Default: FilterContext.INCLUDE.
        op (Union[Unset, FilterOp]):  Default: FilterOp.OR.
        precision (Union[Unset, FilterPrecision]):  Default: FilterPrecision.EXACT.
        q (Union[List[str], Unset, str]):
        daterange (Union[Unset, str]):
        uids (Union[Unset, str]):
        uid (Union[Unset, str]):
    """

    type: str
    context: Union[Unset, FilterContext] = FilterContext.INCLUDE
    op: Union[Unset, FilterOp] = FilterOp.OR
    precision: Union[Unset, FilterPrecision] = FilterPrecision.EXACT
    q: Union[List[str], Unset, str] = UNSET
    daterange: Union[Unset, str] = UNSET
    uids: Union[Unset, str] = UNSET
    uid: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        context: Union[Unset, str] = UNSET
        if not isinstance(self.context, Unset):
            context = self.context.value

        op: Union[Unset, str] = UNSET
        if not isinstance(self.op, Unset):
            op = self.op.value

        precision: Union[Unset, str] = UNSET
        if not isinstance(self.precision, Unset):
            precision = self.precision.value

        q: Union[List[str], Unset, str]
        if isinstance(self.q, Unset):
            q = UNSET
        elif isinstance(self.q, list):
            q = self.q

        else:
            q = self.q

        daterange = self.daterange

        uids = self.uids

        uid = self.uid

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "type": type,
            }
        )
        if context is not UNSET:
            field_dict["context"] = context
        if op is not UNSET:
            field_dict["op"] = op
        if precision is not UNSET:
            field_dict["precision"] = precision
        if q is not UNSET:
            field_dict["q"] = q
        if daterange is not UNSET:
            field_dict["daterange"] = daterange
        if uids is not UNSET:
            field_dict["uids"] = uids
        if uid is not UNSET:
            field_dict["uid"] = uid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type")

        _context = d.pop("context", UNSET)
        context: Union[Unset, FilterContext]
        if isinstance(_context, Unset):
            context = UNSET
        else:
            context = FilterContext(_context)

        _op = d.pop("op", UNSET)
        op: Union[Unset, FilterOp]
        if isinstance(_op, Unset):
            op = UNSET
        else:
            op = FilterOp(_op)

        _precision = d.pop("precision", UNSET)
        precision: Union[Unset, FilterPrecision]
        if isinstance(_precision, Unset):
            precision = UNSET
        else:
            precision = FilterPrecision(_precision)

        def _parse_q(data: object) -> Union[List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                q_type_1 = cast(List[str], data)

                return q_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], Unset, str], data)

        q = _parse_q(d.pop("q", UNSET))

        daterange = d.pop("daterange", UNSET)

        uids = d.pop("uids", UNSET)

        uid = d.pop("uid", UNSET)

        filter_ = cls(
            type=type,
            context=context,
            op=op,
            precision=precision,
            q=q,
            daterange=daterange,
            uids=uids,
            uid=uid,
        )

        return filter_



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FilterContext(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    CONTEXT_UNSPECIFIED: _ClassVar[FilterContext]
    CONTEXT_INCLUDE: _ClassVar[FilterContext]
    CONTEXT_EXCLUDE: _ClassVar[FilterContext]

class FilterOperator(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    OPERATOR_UNSPECIFIED: _ClassVar[FilterOperator]
    OPERATOR_AND: _ClassVar[FilterOperator]
    OPERATOR_OR: _ClassVar[FilterOperator]

class FilterType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    TYPE_UNSPECIFIED: _ClassVar[FilterType]
    TYPE_UID: _ClassVar[FilterType]
    TYPE_HAS_TEXT_CONTENTS: _ClassVar[FilterType]
    TYPE_TITLE: _ClassVar[FilterType]
    TYPE_IS_FRONT: _ClassVar[FilterType]
    TYPE_PAGE: _ClassVar[FilterType]
    TYPE_ISSUE: _ClassVar[FilterType]
    TYPE_STRING: _ClassVar[FilterType]
    TYPE_ENTITY: _ClassVar[FilterType]
    TYPE_NEWSPAPER: _ClassVar[FilterType]
    TYPE_DATERANGE: _ClassVar[FilterType]
    TYPE_YEAR: _ClassVar[FilterType]
    TYPE_LANGUAGE: _ClassVar[FilterType]
    TYPE_TYPE: _ClassVar[FilterType]
    TYPE_REGEX: _ClassVar[FilterType]
    TYPE_MENTION: _ClassVar[FilterType]
    TYPE_PERSON: _ClassVar[FilterType]
    TYPE_LOCATION: _ClassVar[FilterType]
    TYPE_TOPIC: _ClassVar[FilterType]
    TYPE_COLLECTION: _ClassVar[FilterType]
    TYPE_OCR_QUALITY: _ClassVar[FilterType]
    TYPE_CONTENT_LENGTH: _ClassVar[FilterType]
    TYPE_COUNTRY: _ClassVar[FilterType]
    TYPE_ACCESS_RIGHT: _ClassVar[FilterType]
    TYPE_PARTNER: _ClassVar[FilterType]
    TYPE_MONTH: _ClassVar[FilterType]
    TYPE_TEXT_REUSE_CLUSTER_SIZE: _ClassVar[FilterType]
    TYPE_TEXT_REUSE_CLUSTER_LEXICAL_OVERLAP: _ClassVar[FilterType]
    TYPE_TEXT_REUSE_CLUSTER_DAY_DELTA: _ClassVar[FilterType]
    TYPE_TEXT_REUSE_CLUSTER: _ClassVar[FilterType]
    TYPE_MENTION_FUNCTION: _ClassVar[FilterType]
    TYPE_NAG: _ClassVar[FilterType]
    TYPE_WIKIDATA_ID: _ClassVar[FilterType]
    TYPE_DATA_DOMAIN: _ClassVar[FilterType]
    TYPE_COPYRIGHT: _ClassVar[FilterType]
    TYPE_SOURCE_TYPE: _ClassVar[FilterType]
    TYPE_SOURCE_MEDIUM: _ClassVar[FilterType]
    TYPE_ORGANISATION: _ClassVar[FilterType]
    TYPE_EMBEDDING: _ClassVar[FilterType]
    TYPE_IMAGE_VISUAL_CONTENT: _ClassVar[FilterType]
    TYPE_IMAGE_TECHNIQUE: _ClassVar[FilterType]
    TYPE_IMAGE_COMMUNICATION_GOAL: _ClassVar[FilterType]
    TYPE_IMAGE_CONTENT_TYPE: _ClassVar[FilterType]

class FilterPrecision(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    PRECISION_UNSPECIFIED: _ClassVar[FilterPrecision]
    PRECISION_EXACT: _ClassVar[FilterPrecision]
    PRECISION_PARTIAL: _ClassVar[FilterPrecision]
    PRECISION_FUZZY: _ClassVar[FilterPrecision]
    PRECISION_SOFT: _ClassVar[FilterPrecision]

class GroupValue(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    GROUPVALUE_UNSPECIFIED: _ClassVar[GroupValue]
    GROUPVALUE_ARTICLES: _ClassVar[GroupValue]
CONTEXT_UNSPECIFIED: FilterContext
CONTEXT_INCLUDE: FilterContext
CONTEXT_EXCLUDE: FilterContext
OPERATOR_UNSPECIFIED: FilterOperator
OPERATOR_AND: FilterOperator
OPERATOR_OR: FilterOperator
TYPE_UNSPECIFIED: FilterType
TYPE_UID: FilterType
TYPE_HAS_TEXT_CONTENTS: FilterType
TYPE_TITLE: FilterType
TYPE_IS_FRONT: FilterType
TYPE_PAGE: FilterType
TYPE_ISSUE: FilterType
TYPE_STRING: FilterType
TYPE_ENTITY: FilterType
TYPE_NEWSPAPER: FilterType
TYPE_DATERANGE: FilterType
TYPE_YEAR: FilterType
TYPE_LANGUAGE: FilterType
TYPE_TYPE: FilterType
TYPE_REGEX: FilterType
TYPE_MENTION: FilterType
TYPE_PERSON: FilterType
TYPE_LOCATION: FilterType
TYPE_TOPIC: FilterType
TYPE_COLLECTION: FilterType
TYPE_OCR_QUALITY: FilterType
TYPE_CONTENT_LENGTH: FilterType
TYPE_COUNTRY: FilterType
TYPE_ACCESS_RIGHT: FilterType
TYPE_PARTNER: FilterType
TYPE_MONTH: FilterType
TYPE_TEXT_REUSE_CLUSTER_SIZE: FilterType
TYPE_TEXT_REUSE_CLUSTER_LEXICAL_OVERLAP: FilterType
TYPE_TEXT_REUSE_CLUSTER_DAY_DELTA: FilterType
TYPE_TEXT_REUSE_CLUSTER: FilterType
TYPE_MENTION_FUNCTION: FilterType
TYPE_NAG: FilterType
TYPE_WIKIDATA_ID: FilterType
TYPE_DATA_DOMAIN: FilterType
TYPE_COPYRIGHT: FilterType
TYPE_SOURCE_TYPE: FilterType
TYPE_SOURCE_MEDIUM: FilterType
TYPE_ORGANISATION: FilterType
TYPE_EMBEDDING: FilterType
TYPE_IMAGE_VISUAL_CONTENT: FilterType
TYPE_IMAGE_TECHNIQUE: FilterType
TYPE_IMAGE_COMMUNICATION_GOAL: FilterType
TYPE_IMAGE_CONTENT_TYPE: FilterType
PRECISION_UNSPECIFIED: FilterPrecision
PRECISION_EXACT: FilterPrecision
PRECISION_PARTIAL: FilterPrecision
PRECISION_FUZZY: FilterPrecision
PRECISION_SOFT: FilterPrecision
GROUPVALUE_UNSPECIFIED: GroupValue
GROUPVALUE_ARTICLES: GroupValue

class DateRange(_message.Message):
    __slots__ = ("to",)
    FROM_FIELD_NUMBER: _ClassVar[int]
    TO_FIELD_NUMBER: _ClassVar[int]
    to: int
    def __init__(self, to: _Optional[int] = ..., **kwargs) -> None: ...

class Filter(_message.Message):
    __slots__ = ("context", "op", "type", "precision", "q", "daterange", "uids")
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    OP_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    Q_FIELD_NUMBER: _ClassVar[int]
    DATERANGE_FIELD_NUMBER: _ClassVar[int]
    UIDS_FIELD_NUMBER: _ClassVar[int]
    context: FilterContext
    op: FilterOperator
    type: FilterType
    precision: FilterPrecision
    q: _containers.RepeatedScalarFieldContainer[str]
    daterange: DateRange
    uids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, context: _Optional[_Union[FilterContext, str]] = ..., op: _Optional[_Union[FilterOperator, str]] = ..., type: _Optional[_Union[FilterType, str]] = ..., precision: _Optional[_Union[FilterPrecision, str]] = ..., q: _Optional[_Iterable[str]] = ..., daterange: _Optional[_Union[DateRange, _Mapping]] = ..., uids: _Optional[_Iterable[str]] = ...) -> None: ...

class SearchQuery(_message.Message):
    __slots__ = ("filters", "group_by")
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    filters: _containers.RepeatedCompositeFieldContainer[Filter]
    group_by: GroupValue
    def __init__(self, filters: _Optional[_Iterable[_Union[Filter, _Mapping]]] = ..., group_by: _Optional[_Union[GroupValue, str]] = ...) -> None: ...

class CollectionRecommenderParameter(_message.Message):
    __slots__ = ("key", "string_value", "number_value", "bool_value")
    class RecommenderParameterId(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        ID_UNSPECIFIED: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
        ID_COUNT_TYPE: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
        ID_MIN_OCCURRENCES: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
        ID_NUMBER_TO_KEEP: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
        ID_REMOVE_FULLY_MENTIONED: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
        ID_NORMALIZE_MAX_SCORE: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
        ID_MARGIN: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
        ID_SCALING_FACTOR: _ClassVar[CollectionRecommenderParameter.RecommenderParameterId]
    ID_UNSPECIFIED: CollectionRecommenderParameter.RecommenderParameterId
    ID_COUNT_TYPE: CollectionRecommenderParameter.RecommenderParameterId
    ID_MIN_OCCURRENCES: CollectionRecommenderParameter.RecommenderParameterId
    ID_NUMBER_TO_KEEP: CollectionRecommenderParameter.RecommenderParameterId
    ID_REMOVE_FULLY_MENTIONED: CollectionRecommenderParameter.RecommenderParameterId
    ID_NORMALIZE_MAX_SCORE: CollectionRecommenderParameter.RecommenderParameterId
    ID_MARGIN: CollectionRecommenderParameter.RecommenderParameterId
    ID_SCALING_FACTOR: CollectionRecommenderParameter.RecommenderParameterId
    KEY_FIELD_NUMBER: _ClassVar[int]
    STRING_VALUE_FIELD_NUMBER: _ClassVar[int]
    NUMBER_VALUE_FIELD_NUMBER: _ClassVar[int]
    BOOL_VALUE_FIELD_NUMBER: _ClassVar[int]
    key: CollectionRecommenderParameter.RecommenderParameterId
    string_value: str
    number_value: int
    bool_value: bool
    def __init__(self, key: _Optional[_Union[CollectionRecommenderParameter.RecommenderParameterId, str]] = ..., string_value: _Optional[str] = ..., number_value: _Optional[int] = ..., bool_value: _Optional[bool] = ...) -> None: ...

class CollectionRecommender(_message.Message):
    __slots__ = ("type", "weight", "parameters", "enabled")
    class RecommenderType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        TYPE_UNSPECIFIED: _ClassVar[CollectionRecommender.RecommenderType]
        TYPE_TIME_RANGE: _ClassVar[CollectionRecommender.RecommenderType]
        TYPE_ENTITIES: _ClassVar[CollectionRecommender.RecommenderType]
        TYPE_TOPICS: _ClassVar[CollectionRecommender.RecommenderType]
        TYPE_TEXT_REUSE_CLUSTERS: _ClassVar[CollectionRecommender.RecommenderType]
    TYPE_UNSPECIFIED: CollectionRecommender.RecommenderType
    TYPE_TIME_RANGE: CollectionRecommender.RecommenderType
    TYPE_ENTITIES: CollectionRecommender.RecommenderType
    TYPE_TOPICS: CollectionRecommender.RecommenderType
    TYPE_TEXT_REUSE_CLUSTERS: CollectionRecommender.RecommenderType
    TYPE_FIELD_NUMBER: _ClassVar[int]
    WEIGHT_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    ENABLED_FIELD_NUMBER: _ClassVar[int]
    type: CollectionRecommender.RecommenderType
    weight: int
    parameters: _containers.RepeatedCompositeFieldContainer[CollectionRecommenderParameter]
    enabled: bool
    def __init__(self, type: _Optional[_Union[CollectionRecommender.RecommenderType, str]] = ..., weight: _Optional[int] = ..., parameters: _Optional[_Iterable[_Union[CollectionRecommenderParameter, _Mapping]]] = ..., enabled: _Optional[bool] = ...) -> None: ...

class CollectionRecommendersSettings(_message.Message):
    __slots__ = ("recommenders",)
    RECOMMENDERS_FIELD_NUMBER: _ClassVar[int]
    recommenders: _containers.RepeatedCompositeFieldContainer[CollectionRecommender]
    def __init__(self, recommenders: _Optional[_Iterable[_Union[CollectionRecommender, _Mapping]]] = ...) -> None: ...



from enum import Enum
from typing import Literal


class ImpressoNamedEntityRecognitionEntityType(str, Enum):
    COMP_DEMONYM = "comp.demonym"
    COMP_FUNCTION = "comp.function"
    COMP_NAME = "comp.name"
    COMP_QUALIFIER = "comp.qualifier"
    COMP_TITLE = "comp.title"
    LOC = "loc"
    LOC_ADD_ELEC = "loc.add.elec"
    LOC_ADD_PHYS = "loc.add.phys"
    LOC_ADM_NAT = "loc.adm.nat"
    LOC_ADM_REG = "loc.adm.reg"
    LOC_ADM_SUP = "loc.adm.sup"
    LOC_ADM_TOWN = "loc.adm.town"
    LOC_FAC = "loc.fac"
    LOC_ORO = "loc.oro"
    LOC_PHYS_ASTRO = "loc.phys.astro"
    LOC_PHYS_GEO = "loc.phys.geo"
    LOC_PHYS_HYDRO = "loc.phys.hydro"
    LOC_UNK = "loc.unk"
    ORG = "org"
    ORG_ADM = "org.adm"
    ORG_ENT = "org.ent"
    ORG_ENT_PRESSAGENCY = "org.ent.pressagency"
    ORG_ENT_PRESSAGENCY_AFP = "org.ent.pressagency.AFP"
    ORG_ENT_PRESSAGENCY_AG = "org.ent.pressagency.ag"
    ORG_ENT_PRESSAGENCY_ANSA = "org.ent.pressagency.ANSA"
    ORG_ENT_PRESSAGENCY_AP = "org.ent.pressagency.AP"
    ORG_ENT_PRESSAGENCY_APA = "org.ent.pressagency.APA"
    ORG_ENT_PRESSAGENCY_ATS_SDA = "org.ent.pressagency.ATS-SDA"
    ORG_ENT_PRESSAGENCY_BELGA = "org.ent.pressagency.Belga"
    ORG_ENT_PRESSAGENCY_CTK = "org.ent.pressagency.CTK"
    ORG_ENT_PRESSAGENCY_DDP_DAPD = "org.ent.pressagency.DDP-DAPD"
    ORG_ENT_PRESSAGENCY_DNB = "org.ent.pressagency.DNB"
    ORG_ENT_PRESSAGENCY_DOMEI = "org.ent.pressagency.Domei"
    ORG_ENT_PRESSAGENCY_DPA = "org.ent.pressagency.DPA"
    ORG_ENT_PRESSAGENCY_EUROPAPRESS = "org.ent.pressagency.Europapress"
    ORG_ENT_PRESSAGENCY_EXTEL = "org.ent.pressagency.Extel"
    ORG_ENT_PRESSAGENCY_HAVAS = "org.ent.pressagency.Havas"
    ORG_ENT_PRESSAGENCY_KIPA = "org.ent.pressagency.Kipa"
    ORG_ENT_PRESSAGENCY_REUTERS = "org.ent.pressagency.Reuters"
    ORG_ENT_PRESSAGENCY_SPK_SMP = "org.ent.pressagency.SPK-SMP"
    ORG_ENT_PRESSAGENCY_STEFANI = "org.ent.pressagency.Stefani"
    ORG_ENT_PRESSAGENCY_TASS = "org.ent.pressagency.TASS"
    ORG_ENT_PRESSAGENCY_UNK = "org.ent.pressagency.unk"
    ORG_ENT_PRESSAGENCY_UP_UPI = "org.ent.pressagency.UP-UPI"
    ORG_ENT_PRESSAGENCY_WOLFF = "org.ent.pressagency.Wolff"
    ORG_ENT_PRESSAGENCY_XINHUA = "org.ent.pressagency.Xinhua"
    PERS = "pers"
    PERS_COLL = "pers.coll"
    PERS_IND = "pers.ind"
    PERS_IND_ARTICLEAUTHOR = "pers.ind.articleauthor"
    PROD = "prod"
    PROD_DOCTR = "prod.doctr"
    PROD_MEDIA = "prod.media"
    TIME = "time"
    TIME_DATE_ABS = "time.date.abs"
    TIME_HOUR_ABS = "time.hour.abs"
    UNK = "unk"

    def __str__(self) -> str:
        return str(self.value)


ImpressoNamedEntityRecognitionEntityTypeLiteral = Literal[
    "comp.demonym",
    "comp.function",
    "comp.name",
    "comp.qualifier",
    "comp.title",
    "loc",
    "loc.add.elec",
    "loc.add.phys",
    "loc.adm.nat",
    "loc.adm.reg",
    "loc.adm.sup",
    "loc.adm.town",
    "loc.fac",
    "loc.oro",
    "loc.phys.astro",
    "loc.phys.geo",
    "loc.phys.hydro",
    "loc.unk",
    "org",
    "org.adm",
    "org.ent",
    "org.ent.pressagency",
    "org.ent.pressagency.AFP",
    "org.ent.pressagency.ag",
    "org.ent.pressagency.ANSA",
    "org.ent.pressagency.AP",
    "org.ent.pressagency.APA",
    "org.ent.pressagency.ATS-SDA",
    "org.ent.pressagency.Belga",
    "org.ent.pressagency.CTK",
    "org.ent.pressagency.DDP-DAPD",
    "org.ent.pressagency.DNB",
    "org.ent.pressagency.Domei",
    "org.ent.pressagency.DPA",
    "org.ent.pressagency.Europapress",
    "org.ent.pressagency.Extel",
    "org.ent.pressagency.Havas",
    "org.ent.pressagency.Kipa",
    "org.ent.pressagency.Reuters",
    "org.ent.pressagency.SPK-SMP",
    "org.ent.pressagency.Stefani",
    "org.ent.pressagency.TASS",
    "org.ent.pressagency.unk",
    "org.ent.pressagency.UP-UPI",
    "org.ent.pressagency.Wolff",
    "org.ent.pressagency.Xinhua",
    "pers",
    "pers.coll",
    "pers.ind",
    "pers.ind.articleauthor",
    "prod",
    "prod.doctr",
    "prod.media",
    "time",
    "time.date.abs",
    "time.hour.abs",
    "unk",
]



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="DataProviderNamesItem")


@_attrs_define
class DataProviderNamesItem:
    """
    Attributes:
        lang_code (str): ISO 639-1 language code.
        name (str): Name of the data provider in this language.
    """

    lang_code: str
    name: str

    def to_dict(self) -> Dict[str, Any]:
        lang_code = self.lang_code

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "langCode": lang_code,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        lang_code = d.pop("langCode")

        name = d.pop("name")

        data_provider_names_item = cls(
            lang_code=lang_code,
            name=name,
        )

        return data_provider_names_item



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.wikidata_location_type import WikidataLocationType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.wikidata_location_coordinates import WikidataLocationCoordinates
    from ..models.wikidata_location_descriptions import WikidataLocationDescriptions
    from ..models.wikidata_location_labels import WikidataLocationLabels


T = TypeVar("T", bound="WikidataLocation")


@_attrs_define
class WikidataLocation:
    """Wikidata location schema. Based on https://schema.org/Place

    Attributes:
        id (str): The Q Wikidata ID of the location (https://www.wikidata.org/wiki/Wikidata:Identifiers)
        type (WikidataLocationType): The type of the entity
        labels (Union[Unset, WikidataLocationLabels]): Labels of the location in different languages
        descriptions (Union[Unset, WikidataLocationDescriptions]): Descriptions of the location in different languages
        coordinates (Union[Unset, WikidataLocationCoordinates]):
    """

    id: str
    type: WikidataLocationType
    labels: Union[Unset, "WikidataLocationLabels"] = UNSET
    descriptions: Union[Unset, "WikidataLocationDescriptions"] = UNSET
    coordinates: Union[Unset, "WikidataLocationCoordinates"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        type = self.type.value

        labels: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels.to_dict()

        descriptions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.descriptions, Unset):
            descriptions = self.descriptions.to_dict()

        coordinates: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.coordinates, Unset):
            coordinates = self.coordinates.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "type": type,
            }
        )
        if labels is not UNSET:
            field_dict["labels"] = labels
        if descriptions is not UNSET:
            field_dict["descriptions"] = descriptions
        if coordinates is not UNSET:
            field_dict["coordinates"] = coordinates

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.wikidata_location_coordinates import WikidataLocationCoordinates
        from ..models.wikidata_location_descriptions import WikidataLocationDescriptions
        from ..models.wikidata_location_labels import WikidataLocationLabels

        d = src_dict.copy()
        id = d.pop("id")

        type = WikidataLocationType(d.pop("type"))

        _labels = d.pop("labels", UNSET)
        labels: Union[Unset, WikidataLocationLabels]
        if isinstance(_labels, Unset):
            labels = UNSET
        else:
            labels = WikidataLocationLabels.from_dict(_labels)

        _descriptions = d.pop("descriptions", UNSET)
        descriptions: Union[Unset, WikidataLocationDescriptions]
        if isinstance(_descriptions, Unset):
            descriptions = UNSET
        else:
            descriptions = WikidataLocationDescriptions.from_dict(_descriptions)

        _coordinates = d.pop("coordinates", UNSET)
        coordinates: Union[Unset, WikidataLocationCoordinates]
        if isinstance(_coordinates, Unset):
            coordinates = UNSET
        else:
            coordinates = WikidataLocationCoordinates.from_dict(_coordinates)

        wikidata_location = cls(
            id=id,
            type=type,
            labels=labels,
            descriptions=descriptions,
            coordinates=coordinates,
        )

        return wikidata_location



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="WikidataLocationCoordinates")


@_attrs_define
class WikidataLocationCoordinates:
    """
    Attributes:
        latitude (Union[Unset, float]): The latitude of the location
        longitude (Union[Unset, float]): The longitude of the location
    """

    latitude: Union[Unset, float] = UNSET
    longitude: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        latitude = self.latitude

        longitude = self.longitude

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if latitude is not UNSET:
            field_dict["latitude"] = latitude
        if longitude is not UNSET:
            field_dict["longitude"] = longitude

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        latitude = d.pop("latitude", UNSET)

        longitude = d.pop("longitude", UNSET)

        wikidata_location_coordinates = cls(
            latitude=latitude,
            longitude=longitude,
        )

        return wikidata_location_coordinates



from enum import Enum
from typing import Literal


class GetTrPassagesFacetId(str, Enum):
    COLLECTION = "collection"
    CONNECTEDCLUSTERS = "connectedClusters"
    COUNTRY = "country"
    DATERANGE = "daterange"
    LANGUAGE = "language"
    LOCATION = "location"
    NAG = "nag"
    NEWSPAPER = "newspaper"
    ORGANISATION = "organisation"
    PERSON = "person"
    TEXTREUSECLUSTER = "textReuseCluster"
    TEXTREUSECLUSTERDAYDELTA = "textReuseClusterDayDelta"
    TEXTREUSECLUSTERLEXICALOVERLAP = "textReuseClusterLexicalOverlap"
    TEXTREUSECLUSTERSIZE = "textReuseClusterSize"
    TOPIC = "topic"
    TYPE = "type"
    YEAR = "year"

    def __str__(self) -> str:
        return str(self.value)


GetTrPassagesFacetIdLiteral = Literal[
    "collection",
    "connectedClusters",
    "country",
    "daterange",
    "language",
    "location",
    "nag",
    "newspaper",
    "organisation",
    "person",
    "textReuseCluster",
    "textReuseClusterDayDelta",
    "textReuseClusterLexicalOverlap",
    "textReuseClusterSize",
    "topic",
    "type",
    "year",
]



from enum import Enum
from typing import Literal


class GetSearchFacetOrderBy(str, Enum):
    COUNT = "count"
    VALUE = "value"
    VALUE_0 = "-count"
    VALUE_2 = "-value"

    def __str__(self) -> str:
        return str(self.value)


GetSearchFacetOrderByLiteral = Literal[
    "count",
    "value",
    "-count",
    "-value",
]
